var ExamsTutor = (function() {
    var device = ExamsTutorDevicePlugin;
    var localeStrings = {};
    // Start by determining current state

    // Are we logged in?
    if( ! ( device.currentPage('login.html') || device.currentPage('select-pathway.html') ) ) {
        if( device.requiresInitialPathway() ) {
            if( device.requiresLogin()) {
                // If not and we need to be, redirect to login page
                device.showPage('login.html');
                return;
            } else {
                device.showPage('select-pathway.html');
                return;
            }
        }
    }
    // Do we have an active test?
    if( device.activeTest() ) {
    // If so, check that we're trying to display the correct question and ask "Continue or start over" if not.
        // FIXME
    }

    $('.tableviewcell a').live( 'touchstart touchend', toggleTouched );

    function toggleTouched(e) {
        if( e.type == 'touchstart' ) {
            $(this).addClass('touched');
        } else {
            $(this).removeClass('touched');
        }
    }

    function loadLocale( strings ) {
        if( strings ) {
            localeStrings = strings;
        } else {
            localeStrings = {};
        }
    }

    function _(str) {
        if( str in localeStrings) {
            return localeStrings[str];
        } else {
            return str;
        }
    }

    function translateHTMLElements(paths) {
        var elements = [];
        for( var i = 0, l = paths.length; i < l; ++i ) {
            elements = elements.concat($(paths[i]).toArray());
        }
        elements = $(elements);
        elements.each(function() {
            var element = $(this);
            element.html(_(element.text()));
        });
    }

    function retakeTest() {
        var test = ExamsTutor.activeTest();
        if( test['properties'] ) {
            ExamsTutor.startChosenTest(test['properties']);
        } else {
            ExamsTutor.startTest(test['questions'],null);
        }
    }

    function startChosenTest(testProperties) {
        var test_modules = ExamsTutor.getTestModules();
        var questions = test_modules[testProperties['ref']];
        ExamsTutor.startTest(questions,testProperties);
    }

    function startRandomTest(pathway, unit, num) {
        var questions = ExamsTutor.unitTestQuestions(pathway,unit);
        questions = $.shuffle(questions);
        questions = questions.splice(0,num);
        ExamsTutor.startTest(questions,null);
    }

    function startTest(questions, testProperties) {
        var activeTest = {
            currentScore: 0,
            currentQuestion: 0,
            currentMaxScore: 0,
            questions: questions,
            properties: testProperties,
        };
        ExamsTutor.setActiveTest(activeTest);
        ExamsTutor.showPage('question.html');
    }

    function questionAnswered(correct) {
        var test = ExamsTutor.activeTest();
        if(test['properties']) {
            ExamsTutor.savePreviousQuestionResult(test['properties']['ref'],test['currentQuestion']+1,correct);
        } else {
            var question = test['questions'][test['currentQuestion']];
            ExamsTutor.savePreviousQuestionResult(question['testRef'],question['questionNumber'],correct);
        }
        ++test['currentQuestion'];
        ++test['currentMaxScore'];
        if(correct) {
            ++test['currentScore'];
        }
        ExamsTutor.setActiveTest(test);
    }

    function numTestQuestions(pathway) {
        var questions = unitTestQuestions(pathway, null, true);
        return questions.length;
    }

    function unitTestQuestions(pathway, unit, ignoreDemo) {
        var allQuestions = [];
        var test_modules = ExamsTutor.getTestModules();
        var demoTests;
        if( ExamsTutor.demoMode() && ! ignoreDemo) {
            demoTests = ExamsTutor.getAppProperty('demo_tests');
        }
        for( var i = 0, l = pathway['children'].length; i < l; ++i ) {
            if( unit && unit != pathway['children'][i]['id'] ) {
                continue;
            }
            for( var j = 0, jl = pathway['children'][i]['children'].length; j < jl; ++j ) {
                if( demoTests && demoTests.indexOf(Number(pathway['children'][i]['children'][j]['ref'])) == -1 ) {
                    continue;
                }
                questions = test_modules[pathway['children'][i]['children'][j]['ref']];
                for( var k = 0, kl = questions.length; k < kl; ++k ) {
                    questions[k]['questionNumber'] = k+1;
                    questions[k]['testRef'] = pathway['children'][i]['children'][j]['ref'];
                }
                allQuestions = allQuestions.concat(questions);
            }
        }
        return allQuestions;
    }

    function nextQuestion() {
        var test = ExamsTutor.activeTest();
        if( test['currentQuestion'] >= test['questions'].length ) {
            ExamsTutor.saveTestResult(test);
            ExamsTutor.showPage('result.html');
        } else {
            ExamsTutor.showPage('question.html');
        }
    }

    function beginStudyRoomProcess() {
        var username = ExamsTutor.lastLoginUsername();
        var password = ExamsTutor.lastLoginPassword();
        if (username.length == 0)
            username = null;
        if (password.length == 0)
            password = null;
        if (ExamsTutor.demoMode()) {
            ExamsTutor.alert(ExamsTutor._("BUY_ALERT_MESSAGE"), function(){});
        } else if ((username && password) || ExamsTutor.getAppProperty('requires_login')) {
            ExamsTutor.enterStudyRoom();
        } else {
            ExamsTutor.requestStudyRoomDetails();
        }
     }

     var obj = {
        loadLocale: loadLocale,
        _: _,
        translateHTMLElements: translateHTMLElements,
        retakeTest: retakeTest,
        startChosenTest: startChosenTest,
        startRandomTest: startRandomTest,
        startTest: startTest,
        questionAnswered: questionAnswered,
        numTestQuestions: numTestQuestions,
        unitTestQuestions: unitTestQuestions,
        nextQuestion: nextQuestion,
        beginStudyRoomProcess: beginStudyRoomProcess,
        1 : 1 // making sure the last entry has no comma
    };
    for( var key in device ) {
        obj[key] = device[key];
    }
    return obj;
})();
